package org.netkernel.mod.xmldb;

import org.netkernel.layer0.nkf.INKFExpiryFunction;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.util.RequestScopeClassLoader;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Database;
import org.xmldb.api.base.XMLDBException;
/**
 * Collections not threadsafe: http://osdir.com/ml/text.xml.exist/2004-09/msg00030.html
 */
public class XMLDBConfigAccessor extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IHDSNode config=aContext.source("arg:config",IHDSNode.class).getFirstNode("*");
		String collection=aContext.getThisRequest().getArgumentValue("collection");
		
		String driver=config.getFirstValue("xmldbDriver").toString();
		Database database=null;
		for (Database db: DatabaseManager.getDatabases())
		{	if (db.getClass().getName().equals(driver))
			{	database=db;
				break;
			}
		}
		if (database==null)
		{	ClassLoader cl = new RequestScopeClassLoader(aContext.getKernelContext().getRequestScope());
			Class driverClass = cl.loadClass(driver);
			database = (Database)driverClass.newInstance();
			DatabaseManager.registerDatabase(database);
		}
		
		StringBuilder sb=new StringBuilder(256);
		sb.append(config.getFirstValue("xmldbConnection").toString());
		if (!collection.startsWith("/"))
		{	sb.append("/");
		}
		sb.append(collection);
		String uri=sb.toString();
		final Collection result = DatabaseManager.getCollection(uri);
		//TODO authenticated getCollection
		if (result!=null)
		{
			INKFResponse resp=aContext.createResponseFrom(result);
			/*
			INKFExpiryFunction e = new INKFExpiryFunction()
			{	public boolean isExpired(long aNow)
				{	boolean expired;
					try
					{	expired=!result.isOpen();
					}
					catch (XMLDBException e)
					{	expired=true;
					}
					return expired;
				}
			};
			resp.setExpiry(INKFResponse.EXPIRY_FUNCTION, e);
			*/
			resp.setNoCache();
		}
		else
		{
		}
		
	}
	
	
	
}
