package org.netkernel.mod.xmldb;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.w3c.dom.Document;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Resource;
import org.xmldb.api.base.Service;
import org.xmldb.api.modules.CollectionManagementService;
import org.xmldb.api.modules.XMLResource;

public class XMLDBSchemeAccessor extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		//Class requestedRep=aContext.getThisRequest().getRepresentationClass();
		String path=aContext.getThisRequest().getArgumentValue("path");
		int i=path.lastIndexOf("/");
		String collection=path.substring(0,i);
		String resource=path.substring(i+1);
		
		Collection c=Utils.getCollection(aContext, collection);
		if (c==null)
		{	throw new Exception("collection not found");
		}
		
		INKFResponse resp;
		if (resource.length()>0)
		{
			XMLResource r=(XMLResource)c.getResource(resource);
			if (r==null)
			{	throw new Exception("resource not found");
			}
			//TODO handle SAX
			resp=aContext.createResponseFrom(r.getContentAsDOM());
		}
		else
		{	//TODO representation of collection
			HDSBuilder b=new HDSBuilder();
			b.pushNode("collection");
			b.addNode("name", c.getName());
			Collection pc=c.getParentCollection();
			if (pc!=null)
			{	b.addNode("parentCollection",pc.getName());
			}

			b.pushNode("services");
			for (Service service: c.getServices())
			{	b.pushNode("service");
				b.addNode("name", service.getName());
				b.addNode("version", service.getVersion());
				b.popNode();
			}
			b.popNode();
			b.pushNode("childCollections");
			for (String cc: c.listChildCollections())
			{	b.addNode("collection", cc);
			}
			b.popNode();
			b.pushNode("resources");
			for (String resourceName : c.listResources())
			{	b.addNode("resource",resourceName);
			}
			b.popNode();
			
			resp=aContext.createResponseFrom(b.getRoot());
			//throw new Exception("collection listing not implemented yet");
		}
		resp.setNoCache();
	}
	
	public void onSink(INKFRequestContext aContext) throws Exception
	{
		String path=aContext.getThisRequest().getArgumentValue("path");
		int i=path.lastIndexOf("/");
		String collection=path.substring(0,i);
		String resource=path.substring(i+1);
		
		if (resource.length()>0)
		{
			Collection c=Utils.getCollection(aContext, collection);
			if (c==null)
			{	c=Utils.createCollection(aContext,collection);
			}
			XMLResource r=(XMLResource)c.createResource(resource, "XMLResource");
			Document d=aContext.sourcePrimary(Document.class);
			r.setContentAsDOM(d);
			c.storeResource(r);
		}
		else
		{	throw new Exception("sink to collection not implemented");
		}
	}
	
	public void onExists(INKFRequestContext aContext) throws Exception
	{
		String path=aContext.getThisRequest().getArgumentValue("path");
		int i=path.lastIndexOf("/");
		String collection=path.substring(0,i);
		String resource=path.substring(i+1);
		
		boolean result;
		if (resource.length()>0)
		{
			Collection c=Utils.getCollection(aContext, collection);
			result=c.getResource(resource)!=null;
		}
		else
		{	result=Utils.getCollection(aContext, collection)!=null;
		}
		aContext.createResponseFrom(result).setNoCache();
	}
	
	public void onDelete(INKFRequestContext aContext) throws Exception
	{
		String path=aContext.getThisRequest().getArgumentValue("path");
		int i=path.lastIndexOf("/");
		String collection=path.substring(0,i);
		String resource=path.substring(i+1);
		
		if (resource.length()>0)
		{
			Collection c=Utils.getCollection(aContext, collection);
			Resource r=c.getResource(resource);
			c.removeResource(r);
		}
		else
		{	Utils.removeCollection(aContext, collection);
		}
	}
}
